# Game Time sample code
# http://www.willmcgugan.com

SCREEN_SIZE = (640, 480)

# Bounciness of the balls (increase for more bounciness, but dont go over 1!
BOUNCINESS = 0.7

# Game ticks per second
GAME_TICKS = 20

# In pixels per second per second
GRAVITY = 500.0

import pygame
from pygame.locals import *

from random import randint

from gameobjects.vector2 import Vector2
from gameobjects.gametime import GameClock
from gameobjects.util import lerp

# The GameClock instance
game_clock = GameClock(GAME_TICKS)

class Ball(object):
    
    def __init__(self, position, speed, image):
        
        self.position = Vector2(position)
        self.next_position = self.position
        self.speed = Vector2(speed)
        self.image = image
        
        
    def update(self):                
       
        # Calculates the position of the ball at the next game tick
        
        sprite_size = Vector2( self.image.get_size() )
        screen_width, screen_height = SCREEN_SIZE
        
        self.position = self.next_position        
        self.next_position = self.position.copy()       
        
        x, y = self.position - sprite_size / 2 
        w, h = sprite_size
        
        if y + h >= screen_height:
            self.speed.y = -self.speed.y * BOUNCINESS
            self.next_position.y = screen_height - h / 2.0         
                   
        if x <= 0:
            self.speed.x = -self.speed.x * BOUNCINESS
            self.next_position.x = w / 2.0
                    
        elif x + w >= screen_width:
            self.speed.x = -self.speed.x * BOUNCINESS
            self.next_position.x = screen_width - w / 2.0        
        
        self.next_position += self.speed * game_clock.game_tick
        self.speed.y += GRAVITY * game_clock.game_tick
        

    
    def render(self, surface):
                        
        # Lerp between the current position and the next position
        sprite_pos = lerp( self.position,
                           self.next_position,
                           game_clock.get_between_frame() )
        
        sprite_size = Vector2( self.image.get_size() )
        render_pos = sprite_pos - sprite_size / 2
        
        surface.blit(self.image, render_pos)


def run():
    
    global game_clock
    global GAME_TICKS
    
    pygame.init()
    screen = pygame.display.set_mode(SCREEN_SIZE, 0)        
    
    pygame.mouse.set_visible(False)
    
    default_font = pygame.font.get_default_font()
    font = pygame.font.SysFont(default_font, 20, False)    
    
    back_image = pygame.image.load("back.jpg").convert()
    ball_image = pygame.image.load("ball.png").convert_alpha()
    mouse_image = pygame.image.load("mousecursor.png").convert_alpha()
    
    
    balls = []
    
    game_clock.start()
    
    while True:
        
        for event in pygame.event.get():
            
            if event.type == QUIT:
                return
            
            if event.type == MOUSEBUTTONDOWN:                
                
                random_speed = ( randint(-400, 400), randint(-300, 0) )
                
                new_ball = Ball( event.pos,
                                 random_speed,
                                 ball_image )
                
                balls.append(new_ball)
                
            if event.type == KEYUP:
                
                if event.key == K_UP:
                    game_clock.speed += 0.1
                    
                elif event.key == K_DOWN:
                    game_clock.speed = max(game_clock.speed - 0.1, 0)
                    
                elif event.key == K_LEFT:
                    # You are unlikely to need to create new game clocks on the fly like this!
                    if GAME_TICKS > 1:
                        GAME_TICKS -= 1
                        game_clock = GameClock(GAME_TICKS)
                        game_clock.start()
                        del balls[:]
                        
                elif event.key == K_RIGHT:                    
                    
                    GAME_TICKS += 1
                    game_clock = GameClock(GAME_TICKS)
                    game_clock.start()
                    del balls[:]
                    
                if event.key == K_ESCAPE:
                    return
        
        screen.blit(back_image, (0, 0))
        
        # Update phase
        for frame_count, game_time in game_clock.update():
            
            for ball in balls:
                ball.update()
                
        # Render phase
        for ball in balls:
            ball.render(screen)
            
        fps, average_fps = game_clock.get_fps()
            
        speed_txt = """Game speed: %2.2f%% (up / down)""" % (game_clock.speed*100.0)
        click_text = """Game ticks : %i per second (left / right)""" % (game_clock.game_ticks_per_second)
        fps_text = """Frame rate: %2.1f per second""" % (average_fps)
        
        white = (255, 255, 255)
        speed_surface = font.render(speed_txt, True, white)
        screen.blit(speed_surface, (0, 0))
        
        click_surface = font.render(click_text, True, white)
        screen.blit(click_surface, (0, 30))
        
        fps_surface = font.render(fps_text, True, white)
        screen.blit(fps_surface, (0, 60))
            
        mouse_pos = pygame.mouse.get_pos()        
        screen.blit(mouse_image, mouse_pos)
        
        pygame.display.update()
        
if __name__ == "__main__":
    
    run()
